/* $Id$ -*- C++ -*-
 */

#include "contour.h"
#include <cstring>


enum MODE {m0=0, m1=1, m2=2, mINTERFERENCE=-1, mCOMMON=4, mIGNORE=5};

double distance(const Station& s1, const Station& s2)
{
  double R = 6371; // km
  double dLat = s2._latitude - s1._latitude;
  double dLon = s2._longitude - s1._longitude;

  double a = sin(dLat/2) * sin(dLat/2) +
    cos(s1._latitude) * cos(s2._latitude) * sin(dLon/2) * sin(dLon/2); 
  double c = 2 * atan2(sqrt(a), sqrt(1-a)); 
  return R * c;
}

Angle angle(const Station& s1, const Station& s2)
{
  double dLon = s2._longitude - s1._longitude;

  double y = sin(dLon) * cos(s2._latitude);
  double x = cos(s1._latitude)*sin(s2._latitude)-sin(s1._latitude)*cos(s2._latitude)*cos(dLon);
  return atan2(y, x)/DTOR;
}


int main(int argc, char* argv[])
{
  int file_arg = 1;

  if (argc <= file_arg) {
    std::cerr << "no file?\n";
    exit(1);
  }
  
  // <color> ....  RGB color 11DFFF becomes 77FFDF11
  // where the two leftmost digits represent opacity (alpha) values, and
  // where the last six hexadecimal characters represent color as B G R

  std::cout << 
    "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
    "<kml xmlns=\"http://earth.google.com/kml/2.2\">\n"
    "<Document>\n"
    "  <name>" << argv[file_arg] << "</name>\n"
    "  <Style id='c0'>\n"
    "    <LineStyle>\n"
    "      <color>FF0000FF</color>\n"
    "      <width>3</width> <!-- in pixels -->\n"
    "    </LineStyle>\n"
    "  </Style>\n"
    "  <Style id='c1'>\n"
    "    <LineStyle>\n"
    "      <color>FF00FF00</color>\n"
    "      <width>3</width> <!-- in pixels -->\n"
    "    </LineStyle>\n"
    "  </Style>\n"
    "  <Style id='c2'>\n"
    "    <LineStyle>\n"
    "      <color>FFFF0000</color>\n"
    "      <width>3</width> <!-- in pixels -->\n"
    "    </LineStyle>\n"
    "  </Style>\n"
    "  <Style id='c3'>\n"
    "    <LineStyle>\n"
    "      <color>FFFF00FF</color>\n"
    "      <width>3</width> <!-- in pixels -->\n"
    "    </LineStyle>\n"
    "  </Style>\n"
    ;

  for (int ii = file_arg; ii < argc; ++ii) {
    std::ifstream f2(argv[ii]);
    if (!f2) {
      std::cerr << "can't open file: " << argv[ii] << ", skipping\n";
      continue;
    }
    Station s2(f2);

    std::cout <<
      "  <Placemark> \n"
      "    <name>" << argv[ii] << "</name>\n"
      "    <description>\n"
	      << argv[ii] << "\n"
      "    </description>\n"
      "    <Point>\n"
      "      <coordinates>"
	      << (s2._longitude * RTOD) << ","
	      << (s2._latitude * RTOD) << ",0</coordinates>\n"
      "    </Point>\n"
      "  </Placemark>\n"
      ;

    // http://edocket.access.gpo.gov/cfr_2009/octqtr/47cfr73.208.htm
    double ML = s2._latitude;
    double KPDlat = 111.13209 - 0.56605 * cos(2*ML) + 0.00120 * cos(4*ML);
    double KPDlon = 111.41513 * cos(ML) - 0.09455 * cos(3*ML) + 0.00012 * cos(5*ML);

    { // 60 dbu contour
      std::cout <<
	"  <Placemark>\n"
	"    <name>" << argv[ii] << " 60 dBu</name>\n"
	"    <styleUrl>#c0</styleUrl>\n"
	"      <LineString>\n"
	"        <tessellate>1</tessellate>\n"
	"        <altitudeMode>clampToGround</altitudeMode>\n"
	"        <coordinates>\n"
	;
      Station::const_iterator ii = s2._radials.begin();
      bool done = false;
      do {
	if (ii == s2._radials.end()) {
	  ii = s2._radials.begin();
	  done = true;
	}else{
	}
	double d_ew_km = ii->_p60 * sin(ii->_angle.radians());
	double longitude = s2._longitude * RTOD + d_ew_km / KPDlon;
	
	double d_ns_km = ii->_p60 * cos(ii->_angle.radians());
	double latitude = s2._latitude * RTOD + d_ns_km / KPDlat;
	
	std::cout << "<!--" << ii->_angle << "--> " 
		  << longitude << "," 
		  << latitude  << ",0\n";
	++ii;
      } while (!done);
      std::cout <<
	"        </coordinates>\n"
	"      </LineString>\n"
	"  </Placemark>\n"
	;
    }
    { // 54 dbu contour
      std::cout <<
	"  <Placemark>\n"
	"    <name>" << argv[ii] << " 54 dBu</name>\n"
	"    <styleUrl>#c1</styleUrl>\n"
	"      <LineString>\n"
	"        <tessellate>1</tessellate>\n"
	"        <altitudeMode>clampToGround</altitudeMode>\n"
	"        <coordinates>\n"
	;
      Station::const_iterator ii = s2._radials.begin();
      bool done = false;
      do {
	if (ii == s2._radials.end()) {
	  ii = s2._radials.begin();
	  done = true;
	}else{
	}
	double d_ew_km = ii->_i54 * sin(ii->_angle.radians());
	double longitude = s2._longitude * RTOD + d_ew_km / KPDlon;
	
	double d_ns_km = ii->_i54 * cos(ii->_angle.radians());
	double latitude = s2._latitude * RTOD + d_ns_km / KPDlat;
	
	std::cout << "<!--" << ii->_angle << "--> " 
		  << longitude << "," 
		  << latitude  << ",0\n";
	++ii;
      } while (!done);
      std::cout <<
	"        </coordinates>\n"
	"      </LineString>\n"
	"  </Placemark>\n"
	;
    }
    { // 40 dbu contour
      std::cout <<
	"  <Placemark>\n"
	"    <name>" << argv[ii] << " 40 dBu</name>\n"
	"    <styleUrl>#c2</styleUrl>\n"
	"      <LineString>\n"
	"        <tessellate>1</tessellate>\n"
	"        <altitudeMode>clampToGround</altitudeMode>\n"
	"        <coordinates>\n"
	;
      Station::const_iterator ii = s2._radials.begin();
      bool done = false;
      do {
	if (ii == s2._radials.end()) {
	  ii = s2._radials.begin();
	  done = true;
	}else{
	}
	double d_ew_km = ii->_i40 * sin(ii->_angle.radians());
	double longitude = s2._longitude * RTOD + d_ew_km / KPDlon;
	
	double d_ns_km = ii->_i40 * cos(ii->_angle.radians());
	double latitude = s2._latitude * RTOD + d_ns_km / KPDlat;
	
	std::cout << "<!--" << ii->_angle << "--> " 
		  << longitude << "," 
		  << latitude  << ",0\n";
	++ii;
      } while (!done);
      std::cout <<
	"        </coordinates>\n"
	"      </LineString>\n"
	"  </Placemark>\n"
	;
    }
    { // 100 dbu contour
      std::cout <<
	"  <Placemark>\n"
	"    <name>" << argv[ii] << " 100 dBu</name>\n"
	"    <styleUrl>#c3</styleUrl>\n"
	"      <LineString>\n"
	"        <tessellate>1</tessellate>\n"
	"        <altitudeMode>clampToGround</altitudeMode>\n"
	"        <coordinates>\n"
	;
      Station::const_iterator ii = s2._radials.begin();
      bool done = false;
      do {
	if (ii == s2._radials.end()) {
	  ii = s2._radials.begin();
	  done = true;
	}else{
	}
	double d_ew_km = ii->_i100 * sin(ii->_angle.radians());
	double longitude = s2._longitude * RTOD + d_ew_km / KPDlon;
	
	double d_ns_km = ii->_i100 * cos(ii->_angle.radians());
	double latitude = s2._latitude * RTOD + d_ns_km / KPDlat;
	
	std::cout << "<!--" << ii->_angle << "--> " 
		  << longitude << "," 
		  << latitude  << ",0\n";
	++ii;
      } while (!done);
      std::cout <<
	"        </coordinates>\n"
	"      </LineString>\n"
	"  </Placemark>\n"
	;
    }

  }
  std::cout <<
    "</Document>\n"
    "</kml>\n"
    ;

  return 0;
}
